package uni::perl;

our $VERSION = '0.91';

BEGIN {
	defined ${^WARNING_BITS}
		?	${^WARNING_BITS} ^= ${^WARNING_BITS} ^ "\xfc\x3f\xf3\x00\x0f\xf3\xcf\xc0\xf3\xfc\x33\x03\x3f\x00\x0c\x00\x00"
		:	${^WARNING_BITS} = "\xfc\x3f\xf3\x00\x0f\xf3\xcf\xc0\xf3\xfc\x33\x03\x3f\x00\x0c\x00\x00";
	$^H |= 0x1c8e0ee2;

}
m{
use strict;
use warnings;
}x;
use mro ();

# paste this into perl to find bitmask

# no warnings;
# use warnings qw(FATAL closed threads internal debugging pack substr malloc unopened portable prototype
#                 inplace io pipe unpack regexp deprecated exiting glob digit printf
#                 utf8 layer reserved parenthesis taint closure semicolon);
# no warnings qw(exec newline);
# BEGIN { warn join "", map "\\x$_", unpack "(H2)*", ${^WARNING_BITS}; exit 0 };

BEGIN {
	for my $sub (qw(carp croak confess)) {
		no strict 'refs';
		*$sub = sub {
			my $caller = caller;
			local *__ANON__ = $caller .'::'. $sub;
			require Carp;
			*{ $caller.'::'.$sub } = \&{ 'Carp::'.$sub };
			goto &{ 'Carp::'.$sub };
		};
	}
}

sub import {
	my $me = shift;
	my $caller = caller;
	local $^W;
	# use warnings
	${^WARNING_BITS} ^= ${^WARNING_BITS} ^ "\xfc\x3f\xf3\x00\x0f\xf3\xcf\xc0\xf3\xfc\x33\x03\x3f\x00\x0c\x00\x00";

	# use strict, utf8, open, 5.010, ...
	$^H |= 0x1c8e0ee2;

	# use feature qw(feature_state feature_say open> open< feature_arybase feature_switch feature_unicode)
	$^H{'feature_state'} = 1;
	$^H{'feature_say'} = 1;
	$^H{'open>'} = ":utf8";
	$^H{'open<'} = ":utf8";
	$^H{'feature_arybase'} = 1;
	$^H{'feature_switch'} = 1;
	$^H{'feature_unicode'} = 1;

	# use mro 'c3';
	mro::set_mro($caller, 'c3');
	
	#use open (:utf8 :std);
	${^OPEN} = ":utf8\0:utf8";
	binmode(STDIN,   ":utf8");
	binmode(STDOUT,  ":utf8");
	binmode(STDERR,  ":utf8");
	
	for my $sub (qw(carp croak confess)) {
		no strict 'refs';
		*{ $caller .'::'. $sub } = \&$sub;
	}
	return if $caller =~ /^$me/;
	while (@_) {
		my $feature = shift;
		if ($feature =~ s/^://) {
			my $package = $me. '::'. $feature;
			eval "require $package; 1" or croak( "$@" );
			$package->load( $caller );
		}
	}
	require uni::perl::utf;
	uni::perl::utf->load($caller);
	return;
}

sub unimport {
	my $me = shift;
	my $caller = caller;
	# no warnings
	${^WARNING_BITS} = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";

	# no strict, utf8, open, 5.010, ...
	$^H &= ~0x1c8e0ee2;

	# no feature qw(feature_state feature_say open> open< feature_arybase feature_switch feature_unicode)
	delete $^H{'feature_state'};
	delete $^H{'feature_say'};
	delete $^H{'open>'};
	delete $^H{'open<'};
	delete $^H{'feature_arybase'};
	delete $^H{'feature_switch'};
	delete $^H{'feature_unicode'};

	${^OPEN} = ":raw\0:raw";
	binmode(STDIN,   ":raw");
	binmode(STDOUT,  ":raw");
	binmode(STDERR,  ":raw");
	return;
}

1;

__END__

=head1 NAME

uni::perl - all modern features + unicode support in one pragma

=head1 SYNOPSIS

There is most commonly used pragmas, such as L<strict>, L<feature>, L<warnings>, L<utf8>, L<open>, L<mro>. Also almost every modern program uses Encode and Carp

    use uni::perl;

    # is the same as

    use strict;
    use feature qw(say state switch);
    no warnings;
    use warnings qw(FATAL closed threads internal debugging pack substr malloc
                    unopened portable prototype inplace io pipe unpack regexp
                    deprecated exiting glob digit printf utf8 layer
                    reserved parenthesis taint closure semicolon);
    no warnings qw(exec newline);
    use utf8;
    use open (:utf8 :std);
    use mro 'c3';
    use Carp qw(carp croak confess);

    # With ':dumper' you got dumper() function, that similar to Data::Dumper::Dumper()
    use uni::perl ':dumper';

    # With ':xd' you got xd() function from Devel::Hexdump (if installed)
    use uni::perl ':xd';

=cut


=head1 AUTHOR

Mons Anderson, C<< <mons at cpan.org> >>

=head1 THANKS

Thanks to Marc Lehmann for useful hints, found in L<common::sense>

=head1 COPYRIGHT & LICENSE

Copyright 2009-2012 Mons Anderson

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1; # End of uni::perl
